-- constants
Wired.Math.Sphere.Constants = {
  X = 1,
  Y = 2,
  Z = 3,
  RandomizationScale = 2,
};

-- vector functions
Wired.Math.Sphere.GetX = function(vector)
  return vector:Get(Wired.Math.Sphere.Constants.X);
end

Wired.Math.Sphere.GetY = function(vector)
  return vector:Get(Wired.Math.Sphere.Constants.Y);
end

Wired.Math.Sphere.GetZ = function(vector)
  return vector:Get(Wired.Math.Sphere.Constants.Z);
end

-- trigonometry functions
Wired.Math.Sphere.Sin = function(theta)
  return math.sin(math.rad(theta));
end

Wired.Math.Sphere.Cos = function(theta)
  return math.cos(math.rad(theta));
end

Wired.Math.Sphere.Tan = function(theta)
  return math.tan(math.rad(theta));
end

-- point conversion
Wired.Math.Sphere.To3D = function(radius, theta, phi)
  return Wired.Math.Vector({
    (radius * Wired.Math.Sphere.Sin(theta) * Wired.Math.Sphere.Cos(phi)),
    (radius * Wired.Math.Sphere.Sin(theta) * Wired.Math.Sphere.Sin(phi)),
    (radius * Wired.Math.Sphere.Cos(theta))
  });
end

-- rotation matricies

-- rotate y-axis toward z-axis
Wired.Math.Sphere.RotateX = function(theta)
  local sin = Wired.Math.Sphere.Sin(theta);
  local cos = Wired.Math.Sphere.Cos(theta);
  return Wired.Math.Matrix({
    {1, 0, 0},
    {0, cos, -sin},
    {0, sin, cos}
  });
end

-- rotate z-axis toward x-axis
Wired.Math.Sphere.RotateY = function(theta)
  local sin = Wired.Math.Sphere.Sin(theta);
  local cos = Wired.Math.Sphere.Cos(theta);
  return Wired.Math.Matrix({
    {cos, 0, sin},
    {0, 1, 0},
    {-sin, 0, cos}
  });
end

-- rotate x-axis toward y-axis
Wired.Math.Sphere.RotateZ = function(theta)
  local sin = Wired.Math.Sphere.Sin(theta);
  local cos = Wired.Math.Sphere.Cos(theta);
  return Wired.Math.Matrix({
    {cos, -sin, 0},
    {sin, cos, 0},
    {0, 0, 1}
  });
end

-- point distribution

-- golden section distribution
Wired.Math.Sphere.Distribution = function(radius, n)
  local result = {};
  
  -- create random rotation matricies
  local xMatrix = Wired.Math.Sphere.RotateY(math.random(360));
  local yMatrix = Wired.Math.Sphere.RotateX(math.random(360));
    
  -- distribute points
  local increment = math.pi * (3 - math.sqrt(5));
  for i = 1, n do
    local theta = (i - 1) * increment; -- in radians
    local z = (n - (2 * i) + 1) / n;
    local r = math.sqrt(1 - (z * z));
    local vector = Wired.Math.Vector({
      radius * math.cos(theta) * r,
      radius * math.sin(theta) * r,
      radius * z
    });
    
    -- rotate vector
    vector = Wired.Math.Matrix.Multiply(xMatrix, vector);
    vector = Wired.Math.Matrix.Multiply(yMatrix, vector);
    result[i] = vector;
  end
  return result;
end

-- randomize array
Wired.Math.Sphere.Randomize = function(array)
  local scale = #array * Wired.Math.Sphere.Constants.RandomizationScale;
  for i = 1, scale do
    local index1 = math.random(1, #array);
    local index2 = math.random(1, #array);
    if (index1 ~= index2) then
      local value = array[index1];
      array[index1] = array[index2];
      array[index2] = value;
    end
  end
  return array;
end

-- random distribution
Wired.Math.Sphere.RandomDistribution = function(radius, n)
  return Wired.Math.Sphere.Randomize(Wired.Math.Sphere.Distribution(radius, n));
end
